
;============================ENGLISH====================================

;  This is a variation of Get2Date, which lets the user specify a date 
;  from within Mo-Fr. You can include Saturday, too, if you disable 
;  code in proc rkCalcDates. See the comments in that proc.
;  Public Holidays are also excluded, if you have HOLIDAYS.DB in the same
;  directory (and it's stuffed with proper information...). Delete or 
;  rename the table, if you don't want to use it. Up to five holidays
;  per month are possible, if you need more, please tell me where you
;  live and simply expand the array hPos.ay and the corresponding 
;  PAINTCANVAS-commands. 

;  Further setup options: First day of week can be Monday or Sunday 
;  (var mFirst.b), week-of-year can be displayed (var wDisp.b). Again
;  you can speed things up by completely deleting the corresponding lines.
;  See calling example on the end of the script on how to configure.

;-----Dialog is based on "GETDATE", see copyright notice below!-----
;-----------Thanks to Leon Chalnick for uploading GETDATE!----------
;===========================================================================

;===============================DEUTSCH=====================================
;  Variation von Get2Date, der Benutzer kann einen Termin von Mo-Sa waehlen. 
;  Feiertage werden auch ausgeschlossen, wenn die Tabelle FEIERTAG.DB im
;  gleichen Verzeichnis steht (und korrekt gefuettert wurde...). Tabelle 
;  loeschen oder umbenennen, wenn's nicht gewuenscht wird.
;  Fuenf Feiertage pro Monat sind moeglich; wenn mehr gebraucht werden, 
;  bitte ich um Preisgabe dieses paradiesischen Ortes. Ansonsten einfach
;  das array hPos.ay vergroessern und die dazugehoerigen PAINTCANVAS-Befehle
;  ergaenzen.

;  Setup Optionen: Woche kann Montag oder Sonntag beginnen (var mFirst.b),
;  Kalenderwochen (KW) knnen angezeigt werden (var wDisp.b), Text kann 
;  Deutsch oder Englisch sein (var tGer.b). Konfigurationsbeispiel findet 
;  am Ende des Scripts im Aufrufbeispiel.

;-----Der Dialog basiert auf "GETDATE", siehe Copyright unten!-----
;-----------Besten Dank an Leon Chalnick fuers Hochladen!----------
;===========================================================================

;=============================================================================
;Copyright 1992 Professional Information Consulting, Inc., Leon Chalnick
;You are granted the right to use this code in your applications as long as 
;this notice is included.  You are not granted the right to resell this code.
;=============================================================================
;ADDITIONAL COPYRIGHT: Reinhard Kalinke, expansion for delivery-dates
;with custom reminder (e.g. for checks if delivery is on it's way)
;             Copyright conditions as stated above
;   Comments and suggestions: Reinhard Kalinke, CIS 100417,3504
;=============================================================================


PROC dGet1Date(dDfltDate)
;=============================================================================
;AUTHOR:		PIC-LMC, rk
;DATE:		8/27/92 , 8/02/95
;FUNCTION:
;	Displays a calendar, allows user to scroll through days, weeks, months,
;years (using keyboard or mouse) and select a date.  It also provides an
;accept control into which user can directly enter the date (the calendar will
;be synchronized when user hits OK or leaves the accept control).  It provides
;a very friendly mechanism for onbtaining a date from the user. It will be most
;useful when the user would benefit from seeing an entire calendar.
;	This proc is primarily responsible for specifying the d-box. It is the
;d-box proc (dbpGet2Date) that controls the processing of user input.
;	Note that the calendar is displayed through a single paintcanvas command.
;The calendaris actually a formatted text string produced in NewMonth().  It
;gets divided into rows which are stored in a dynamic array.  However, because
;the text string is formatted so that each line of the calendar is exactly the
;same length (and its < 255 chars), the paintcanvas command can paint it out
;over a rectangular area with one command.
;	Note the the 'mouse buttons' used for scrolling through months and years
;are not really buttons, but are produced w/canvas painting commands.  We
;check for mouse clicks on them by trapping the mouse "DOWN" event, and
;localizing it to screen coordinates wihtin the dbox window.
;	Here is the keyboard interface (mouse buttons follow these conventions too):
;	Key				Action
;-----------------------------------------------------------------
;	Left				previous day
;	Right				next day
;	Up				previous week
;	Down				next week
;	PgUp				same day, previous month
;	PgDn				same day, next month
;	Ctrl+PgUp		        same day, previous year
;	Ctrl+PgDn		        same day, next year
;
;       -                               decreases remiShift.n
;       +                               increases remiShift.n
;
;	Enter				Ok
;	Esc				Cancel
;
;PARAMS:
;	dDfltDate - the date to which calendar will default.
;CALLS:
;	SetUpCal   - Sets up initial display of calander
;	dbpGet2Date - d-box proc for this dialog box.
;RETURNS:
;	If user hit Ok, returns dCdate, the currently selected date.  If user
;hit cancel, returns a blankdate.
;
;=============================================================================
private
;Regional vars--will be used and/or initialized by one or more called procs:
hDBox,		;Handle to dBox window
lOk,		;True if user exited with Ok button, false if user exits w/Cancel
nCRow,		;Cursor row
nCCol,		;Cursor col
nOldCol,	;Last cursor column (used to paint over previous cursor location)
nOldRow,	;Last cursor row (used to paint over previous cursor location)
a,		;Text string containing entire formatted list of days
DYCAL,		;Dynarray with formatted callendar (variable a, sliced up)
nCalRows,	;Number of rows in calander
nBtnClr,	;Pdox color for dbox buttons, used on Month/Year 'mouse buttons'
nDBoxHlt,       ;color for 3d Box appearance
nDBoxClr,       ;color for text display
nDAccClr,       ;color for text preceeding accept-controls
dowAccClr,      ;color for dow preceeding delivery accept
dowAccRemClr,   ;color for dow preceeding reminder accept
nDSunClr,       ;color for sundays and public holidays
                ;all these colors are set up in proc SetUpCal!
dCdate,		;Date that cursor is on-return value
aMth,		;Month of dCdate (for mouse button display)
aYr,		;Year of dCdate (for mouse button display)
oMth,		;Old month, to check if calendar has to be updated
oYr,		;Old year, to check if calendar has to be updated 
dowLong.a,      ;long text for today's dow
realRS.b,       ;true if remiShift.n is really applied,
                ;false when falling short due to proc rkCalcDates
                ;(paints corresponding text red then)
dAcptDate,      ;temp delivery date
iGer.b,         ;'internal' language of Pdox/TP, true for german
;Local vars--Only used in this proc-----------------------------------
aOkCancel	;Control var for OK Cancel buttons

lOk=false
realRS.b=true

SetUpCal()	;Set up initial calendar display

; rk_Color_Setup_Begin
; this color deviation lets labels look like buttons:
; (names are changed because PaintPAL used to reject this code)
GETCOLORS TO rk.OriginalPalette
DYNARRAY rk.ColorDeviations[]
rk.ColorDeviations[1037] = rk.OriginalPalette[1040] ; 23  ;Label Inactive
rk.ColorDeviations[1038] = rk.OriginalPalette[1042] ; 31  ;Label Selected
rk.ColorDeviations[1039] = rk.OriginalPalette[1044] ; 27  ;Label Shortcut
SETCOLORS FROM rk.ColorDeviations
RELEASE VARS rk.ColorDeviations
; rk_Color_Setup_End

; PaintPAL_Generated_Code_Begin(520263506)

SHOWDIALOG iif(tGer.b,dowLong.a+", der "+strval(today()),dowLong.a+", "+strval(today()))  ; C:\PDOX45\EBS\GETDATE\GET1DATF.DLG
   PROC "dbpGet2Date"
      MOUSE "DOWN", "AUTO"
      KEY "+", "-", "CTRLPGDN", "CTRLPGUP", "DOWN", "LEFT", "PGDN", "PGUP", "RIGHT", "UP"
      TRIGGER "OPEN", "ARRIVE", "UPDATE", "DEPART", "ACCEPT"

   @3,21 HEIGHT 17 WIDTH 30

   PAINTCANVAS FILL a ATTRIBUTE nDBoxClr 2,3,nCalRows+2,23

   FRAME SINGLE FROM 1,3 TO 1+nCalRows+1,24

   FRAME SINGLE FROM 10,3 TO 12,24

   @1,4
   ?? iif(mFirst.b,iif(tGer.b,"Mo-Di-Mi-Do-Fr-Sa-So","Mo-Tu-We-Th-Fr-Sa-Su"),iif(tGer.b,"So-Mo-Di-Mi-Do-Fr-Sa","Su-Mo-Tu-We-Th-Fr-Sa"))

   @9,4
   ?? chr(27)+" "+aMth+" "+chr(26)+"     "+chr(27)+" "+aYr+" "+chr(26)

   ; PaintPAL_Static_Text_Begin
   PAINTCANVAS FILL " " ATTRIBUTE dowAccClr 11,4,11,9
   @11,4
   ?? "   "+format("W2,ar",dow(dAcptDate))+","
   PAINTCANVAS ATTRIBUTE dowAccClr 11,4,11,9
   ; PaintPAL_Static_Text_End

   @1,1
   ?? iif(wDisp.b,iif(tGer.b,"KW","Wk"),"")
   @2,1
   ?? woy.ay[1]
   @3,1
   ?? woy.ay[2]
   @4,1
   ?? woy.ay[3]
   @5,1
   ?? woy.ay[4]
   @6,1
   ?? woy.ay[5]
   @7,1
   ?? woy.ay[6]

   PAINTCANVAS FILL " " ATTRIBUTE nDBoxClr nCalRows+2,1,nCalRows+3,2

   PAINTCANVAS ATTRIBUTE nDWeekClr 1,1,nCalRows+2,2

   PAINTCANVAS ATTRIBUTE nDBoxHlt 10,3,10,23

   PAINTCANVAS ATTRIBUTE nDBoxClr 12,3,12,23

   PAINTCANVAS ATTRIBUTE nDBoxHlt 10,3,12,3

   PAINTCANVAS ATTRIBUTE nDBoxClr 10,24,11,24

   PAINTCANVAS ATTRIBUTE nDBoxHlt 1,3,1+nCalRows+1,3

   PAINTCANVAS ATTRIBUTE nDBoxHlt 1,3,1,23

   PAINTCANVAS ATTRIBUTE nBtnClr 9,3,9,24

   PAINTCANVAS ATTRIBUTE nDBoxClr 9,6,9,8

   PAINTCANVAS ATTRIBUTE nDboxClr 9,12,9,14

   PAINTCANVAS ATTRIBUTE nDBoxClr 9,18,9,21

   PAINTCANVAS ATTRIBUTE nDBoxClr 1,4,1,5

   PAINTCANVAS ATTRIBUTE nDBoxClr 1,7,1,8

   PAINTCANVAS ATTRIBUTE nDBoxClr 1,10,1,11

   PAINTCANVAS ATTRIBUTE nDBoxClr 1,13,1,14

   PAINTCANVAS ATTRIBUTE nDBoxClr 1,16,1,17

   PAINTCANVAS ATTRIBUTE nDBoxClr 1,19,1,20

   PAINTCANVAS ATTRIBUTE nDBoxClr 1,22,1,23

   PAINTCANVAS ATTRIBUTE nDboxClr nOldRow,nOldCol,nOldRow,nOldCol+1

   PAINTCANVAS ATTRIBUTE nDSunClr hPos.ay[1],hPos.ay[2],hPos.ay[1],hPos.ay[2]+1

   PAINTCANVAS ATTRIBUTE nDSunClr hPos.ay[3],hPos.ay[4],hPos.ay[3],hPos.ay[4]+1

   PAINTCANVAS ATTRIBUTE nDSunClr hPos.ay[5],hPos.ay[6],hPos.ay[5],hPos.ay[6]+1

   PAINTCANVAS ATTRIBUTE nDSunClr hPos.ay[7],hPos.ay[8],hPos.ay[7],hPos.ay[8]+1

   PAINTCANVAS ATTRIBUTE nDSunClr hPos.ay[9],hPos.ay[10],hPos.ay[9],hPos.ay[10]+1

   PAINTCANVAS ATTRIBUTE nDSunClr 2,iif(mFirst.b,22,4),nCalRows+1,iif(mFirst.b,23,5)

   PAINTCANVAS ATTRIBUTE nDSunClr 1,iif(mFirst.b,22,4),1,iif(mFirst.b,23,5)

   PAINTCANVAS ATTRIBUTE nBtnClr nCRow,nCCol,nCRow,nCCol+1

   PUSHBUTTON @13,1 WIDTH 12
      iif(tGer.b," ~W~eiter"," ~O~k")
      OK
      DEFAULT
      VALUE "OK"
      TAG "OK"
      TO aOkCancel

   ACCEPT @11,10 WIDTH 14
      "D"
      REQUIRED
      TAG "ACCEPTDATE"
      TO dAcptDate

   PUSHBUTTON @13,14 WIDTH 13
      iif(tGer.b,"~A~bbruch","~C~ancel")
      CANCEL
      VALUE "CANCEL"
      TAG "CANCEL"
      TO aOkCancel

ENDDIALOG

; PaintPAL_Generated_Code_End(520263506)

; rk_Color_Cleanup_Begin
SETCOLORS FROM rk.OriginalPalette
RELEASE VARS rk.OriginalPalette
; rk_Color_Cleanup_End

return iif(lOk,dCdate,blankdate())
ENDPROC

PROC dbpGet2Date(aType,aTag,vEventVal,vElementVal)
;=============================================================================
;Copyright 1992 Professional Information Consulting, Inc., Leon Chalnick
;You are granted the right to use this code in your applications as long as 
;this notice is included.  You are not granted the right to resell this code.
;=============================================================================
;AUTHOR:		PIC-LMC, rk
;DATE:		8/27/92 / 8/08/95
;FUNCTION:
;	Controls processing of events that we trapped for in d-box spec.  Calls
;appropriate procs for subsequent processing.
;	Keyboard events are easy to process.  To process mouse clicks, we must 
;localize the event to the dbox window and determine what action to take based
;on whcih row/column mouse was clicked in.  NOTE THAT ROW AND COLUMN
;COORDINATES RESULTING FROM THE "localize evnet" COMMAND ARE 1 GREATER THAN THE
;COORDINATES THAT WERE DEFINED IN THE DBOX!
;	Note also that Paradox can't properly proceses events once they have been
;localized to a window (and we want it to exec localized events that are not in
;the coordinates of either the calendar or the 'mouse buttons'; i.e., push
;button and d-box moving events.)  To work around this problem, we'll copy the
;original mouse event from vEventVal to dyEventRec and localize dyEventRec
;instead of vEventVal.  That way, Pdox can still process the original event
;(vEventVal) when this proc returns true.
;
;PARAMS:
;	The normal params for a d-box:  aType is the type of event that called
;the dbox proc.  aTag is the tag of the current control element. vEventVal
;is usually (always in this case) a dynamic array containg a record of the 
;event that resulted in call to d-box proc.  vElementVal will be blank in this
;d-box, but will contain value related to check box, if one is used in d-box.
;
;CALLS:
;	NewDay         - Determines new cursor location for movment of one day
;	IncreaseDate   - Increases date by one DAY, WEEK, MONTH or YEAR
;	DecreaseDate   - Decreases date by one DAY, WEEK, MONTH or YEAR
;
;CALLED BY:
;	dGet1Date
;=============================================================================
private 
nKey,				;If call was for a key event, will contain keycode, otherwise, 0
lMouseClick,            	;True if call was for a mouse down event
lMouseAuto,                     ;True if call was for mouse auto (button pressed)
;---lMouseAuto:-----------------------------------------------------
;PDOX obviously tends to swallow the uplifting of the mouse-button 
;sometimes. Display then keeps running (like a VW-beetle...)
;If you don't want auto-shifting by mouse, remove all occurences of
;lMouseAuto and the MOUSE "AUTO"-event in showdialog.
;It's enabled only for in/decrease of 'reminder days-ahead' and
;disabled for the in/decrease of month and year. To enable that, too,
;remove "and not lMouseAuto" from line 410.
;--------------------------------------------------------------------
nRow,	        	;If call was for mouse down, row localized to dbox widnow
nCol,			;If call was for mouse down, column localized to dbox widnow
nDay,			;If call was for mouse down, day user clicked on (or "Error" if
			;user clicked on blank).
oYr,                    ;old year, in case year changes holidays have to be set
oMth,                   ;old month, in case month changes holidays have to be set
dyEventRec,		;We need to localize the mouse event to this d-box window.  The
			;problem is, Paradox can't execute events that have been 
			;localized, so we copy the original event (vEventVal) into this
			;dynarray and localize THIS dynarray.
			;(GlobalizeEvent wasn't born yet, alter if you
			;want to speed things up a bit - rk)
aEventTag,		;Event tag identifier used in 'foreach loop'
lProcessEvent	        ;Return value

oYr=aYr
oMth=aMth
nKey=iif(aType="EVENT" and vEventVal["TYPE"]="KEY", vEventVal["KEYCODE"], 0)
lMouseClick=iif(aType="EVENT" and vEventVal["TYPE"]="MOUSE", vEventVal["ACTION"]="DOWN", false)
lMouseAuto=iif(aType="EVENT" and vEventVal["TYPE"]="MOUSE", vEventVal["ACTION"]="AUTO", false)
lProcessEvent=true

switch
case aType="ACCEPT":
	lOk=true

case aType="DEPART" and aTag="ACCEPTDATE":
	;The user is departing the accept control.  We'll synchronize the display
	;of the mouse buttons, calendar and the cursor to the new date.  This code
	;is not necessary if user didn't change date, but it's so fast that it is
	;simpler to do it anyway.
        dowAccClr=nDAccClr ;paint dow normal
        rkSyncCal()
	rkCalcDates()
	
case aType="ARRIVE" and aTag="ACCEPTDATE":
        ; paint dow in grey
        dowAccClr = nDAccClr-mod(nDAccClr,16)+7 ;grey        
        refreshdialog

case nKey<>0 and (aTag="ACCEPTDATE"):
	;We'll return true--user is in accept control, but is hitting keys that 
	;we're trapping for.  We'll just let Pdox process them.

case lMouseClick or lMouseAuto:
	;Because localized events aren't properly processed when you return true
	;from the d-box proc, we will copy the event array to a dynarray and
	;localize the event in the copied array.
	dynarray dyEventRec[]
	foreach aEventTag in vEventVal
		dyEventRec[aEventTag]=vEventVal[aEventTag]
	endforeach
	localizeevent dyEventRec
	;Make sure click was in dbox window
	if dyEventRec["WINDOW"]=hDBox
		then nRow=dyEventRec["ROW"]
		nCol=dyEventRec["COL"]
		switch
		case ((nRow>=3 and nRow<=2+nCalRows) and nCol<25) and not lMouseAuto:
			;Check for mouse clicks in calendar itself (on col/row containing a number)
			if search(","+strval(nCol)+",", ",5,6,8,9,11,12,14,15,17,18,20,21,23,24,")<>0
				then nDay=numval(substr(DYCAL[nRow-2],iif(mod(nCol,3)=0, nCol-1-3, nCol-3), 2))
				;If user clicked on a blank location (at beginning or ending of 
				;month), nDay will = Error.
				if nDay<>"Error"
					then dCdate=dateval(strval(month(dCdate))+"/"+strval(nDay)+"/"+strval(year(dCdate)))
					NewDay(dCdate)					;Determine row/col for new day
					;aMth is used in display of month in 'mouse buttons'
                                  	if iGer.b=tGer.b then
                                           aMth=moy(dCDate)
                                        else
                        	           ;debug
                        	           aMth=iif(tGer.b,substr("JanFebMrAprMaiJunJulAugSepOktNovDez",month(dCdate)*3-2,3),
                        	                           substr("JanFebMarAprMayJunJulAugSepOctNovDec",month(dCdate)*3-2,3))
                                        endif   
					dAcptDate=dCdate				;Resync value in accept control
                                     	rkCalcDates()
				endif ;Mouse click location wasn't blank
			endif ;User clicked on column/row that could have a date in it
			lProcessEvent=false
		case nRow=10 and ncol<25 and not lMouseAuto: 
		        ;no Auto-shifting here!
			;Check for clicks on "mouse buttons" for changing months and years
			switch
			case nCol>=4 and nCol<=6:	;Decrease month
				DecreaseDate("MONTH")
			case nCol>=10 and nCol<=12:	;Increase month
				IncreaseDate("MONTH")
			case nCol>=16 and nCol<=18:	;Decrease year
				DecreaseDate("YEAR")
			case nCol>=23 and nCol<=25:	;Increase year
				IncreaseDate("YEAR")
			endswitch
			lProcessEvent=false
		case nRow=10 and ncol>25:
			;Check for clicks for changing WARNDATE
			switch
			case nCol>=29 and nCol<=31:
	                  remiShift.n = remiShift.n-1
			case nCol>=50 and nCol<=52:
	                  remiShift.n = remiShift.n+1
			endswitch
                	rkCalcDates()
			lProcessEvent=false
		otherwise:
			;User clicked mouse within d-box window boundry, but in an area in
			;which we will just let Paradox process the event.
		endswitch ;Row user clicked in
	endif ;User clicked mouse in dbox
;The rest of the events are keyboard events
case nKey=asc("UP"):
	DecreaseDate("WEEK")
case nKey=asc("DOWN"):
	IncreaseDate("WEEK")
case nKey=asc("LEFT"):
	DecreaseDate("DAY")
case nKey=asc("RIGHT"):
	IncreaseDate("DAY")
case nKey=asc("PGUP"):
	DecreaseDate("MONTH")
case nKey=asc("PGDN"):
	IncreaseDate("MONTH")
case nKey=asc("CTRLPGUP"):
	DecreaseDate("YEAR")
case nKey=asc("CTRLPGDN"):
	IncreaseDate("YEAR")
;----------------------------------
case nKey=asc("-"):                 
        remiShift.n = remiShift.n-1  
        rkCalcDates()
case nKey=asc("+"):                 
        remiShift.n = remiShift.n+1 
        rkCalcDates()
;----------------------------------        
case aType="OPEN":
	;Save window handle; it's used to localize mouse events
	window handle dialog to hDbox
	
endswitch

return lProcessEvent

ENDPROC


PROC SetUpCal()
;=============================================================================
;Copyright 1992 Professional Information Consulting, Inc., Leon Chalnick
;You are granted the right to use this code in your applications as long as 
;this notice is included.  You are not granted the right to resell this code.
;=============================================================================
;AUTHOR:		PIC-LMC
;DATE:		8/27/92
;FUNCTION:
;	Sets up initial display of calendar, screen coordinates, other variables
;used throughout process.
;
;CALLED BY:
;	dGet1Date
;CALLS:
;	NewMonth - Initializes vars used in producing calendar
;	NewDay   - Determines new cursor location for movment of one day
;=============================================================================
private
col.dy,		;Temp dynarray for system colors
dow.dy,         ;Temp dynarray for dowLong.a
sys.dy,         ;Temp Dynarray for system-info
holiTbl.a,      ;Name of table holding public holidays
fName.a         ;Name of date field in table holiTbl.a

sysinfo to sys.dy
switch
 case sys.dy["LANGUAGE"] = "001":
  iGer.b = false
 case sys.dy["LANGUAGE"] = "049":
  iGer.b = true
 otherwise:
  message "This program is for use with english or german versions of PDOX only!" 
  quit
endswitch 

;-----Set up the public holidays:------
holiTbl.a = iif(tGer.b,"Feiertag","Holidays") ; Table with public holidays
if istable(holiTbl.a) and not isempty(holiTbl.a) then
   dynarray hDays.dy[]       ;Array fuer Feiertage
   view holiTbl.a
   fName.a = iif(tGer.b,"Datum","Date")
   moveto field fName.a
   scan
     hDays.dy[[]] = [Text]
   endscan  
   clearimage
else
   array hPos.ay[10]     ; blind array, could be done with iif in the 
   for i from 1 to 10    ; corresponding paintcanvas, too...
       hpos.ay[i] = 0    ; (I didn't manage to find a 'smart' syntax 
   endfor                ; for that, you'll have to check each
endif                    ; array-element seperately, or do you...?)

dynarray dow.dy[]
;---German with Pdox:
switch
 case tGer.b and iGer.b:
  dow.dy["Son"] = "Sonntag"
  dow.dy["Mon"] = "Montag"
  dow.dy["Die"] = "Dienstag"
  dow.dy["Mit"] = "Mittwoch"
  dow.dy["Don"] = "Donnerstag"
  dow.dy["Fre"] = "Freitag"
  dow.dy["Sam"] = "Samstag"
;---German with TurboPAL:
 case tGer.b and not iGer.b:
  dow.dy["Sun"] = "Sonntag"
  dow.dy["Mon"] = "Montag"
  dow.dy["Tue"] = "Dienstag"
  dow.dy["Wed"] = "Mittwoch"
  dow.dy["Thu"] = "Donnerstag"
  dow.dy["Fri"] = "Freitag"
  dow.dy["Sat"] = "Samstag"
;---English text with german Pdox:
 case not tGer.b and iGer.b:
  dow.dy["Son"] = "Sunday"
  dow.dy["Mon"] = "Monday"
  dow.dy["Die"] = "Tuesday"
  dow.dy["Mit"] = "Wednesday"
  dow.dy["Don"] = "Thursday"
  dow.dy["Fre"] = "Friday"
  dow.dy["Sam"] = "Saturday"
;---English with Pdox/TurboPAL:
 otherwise:
  dow.dy["Sun"] = "Sunday"
  dow.dy["Mon"] = "Monday"
  dow.dy["Tue"] = "Tuesday"
  dow.dy["Wed"] = "Wednesday"
  dow.dy["Thu"] = "Thursday"
  dow.dy["Fri"] = "Friday"
  dow.dy["Sat"] = "Saturday"
endswitch
dowLong.a = dow.dy[dow(today())]

;----Color setup----
getcolors to col.dy
;System color for dbox 'fake' buttons:
nBtnClr=col.dy["1040"]	
;Dbox text:
nDboxClr=col.dy["1036"]		
;color of text preceeding accepts (dow), same as accept-control:
nDAccClr=col.dy["1049"]
;dow at delivery date Accept:
dowAccClr=nDAccClr
;dow at reminder date Accept:
dowAccRemClr=nDAccClr
;Highlight for 3D dbox appearance (maybe has to be customized...):
nDBoxHlt=nDBoxClr-mod(nDBoxClr,16)+iif(mod(nDBoxClr,16)>8,0,15)
;Sunday colors:
nDSunClr=nDBoxClr-mod(nDBoxClr,16)+4 ;(red)
;color for reminder text if the value can't be applied
nDnRSClr=nDBoxClr-mod(nDBoxClr,16)+4 ;(red)
;week-of-year color:
nDWeekClr=nDBoxClr-mod(nDBoxClr,16)+8 ;(dark grey)

;----calendar setup----
dynarray DYCAL[]	;Dynarray used to hold calendar weeks
if iGer.b=tGer.b then
   aMth=moy(dDfltDate)
else
   aMth=iif(tGer.b,substr("JanFebMrAprMaiJunJulAugSepOktNovDez",month(dDfltDate)*3-2,3),
                   substr("JanFebMarAprMayJunJulAugSepOctNovDec",month(dDfltDate)*3-2,3))
endif   
aYr=strval(year(dDfltDate))
NewMonth(dDfltDate)
;Must set up "current" screen coordinates of cursor.  Will be changed in
;NewDay().
nCRow=2			;Row can't be less than 2
nCCol=4			;Column can't be less than 4
NewDay(dDfltDate)	;Determines actual scrn coord for cursor
dCdate=dDfltDate								;Set current cursor date to dflt date
dAcptDate=dDfltDate	;Resync value in accept control
oYr = ""                ;initialize vars for first setup of holidays
oMth = ""
rkCalcDates()

ENDPROC

PROC NewMonth(dNewDate)
;=============================================================================
;Copyright 1992 Professional Information Consulting, Inc., Leon Chalnick
;You are granted the right to use this code in your applications as long as 
;this notice is included.  You are not granted the right to resell this code.
;=============================================================================
;AUTHOR:		PIC-LMC
;DATE:		8/27/92
;FUNCTION:
;	Sets up screen display of a new month.  This means re-initializing the text
;string , 'a', with the formatted list of days in the month.  It also must 
;split this string into a dynarray (DYCAL[]) and determine the number of rows
;in the calendar.
;  The dynarray is used by NewDay() and dbpGet2Date() in determining the row
;and column for dCdate.
;  The number of calendar rows is used in determining where certain things are
;to be painted and where the mouse was clicked.
;	The text string is produced so that it looks like this:
;
;"SPACES BEFORE 1ST DAY OF MTH"+" nn nn nn ..."+"SPACES AFTER LAST DAY OF MTH"
;
;PARAMS:
;	dNewDate  - This is the new date to which the cursor is being moved.
;CALLED BY:
;	SetUpCal
;	IncreaseDate
;	DecreaseDate
;=============================================================================
private
i,				;Counter in for loops
d1stOfMth,	;Date of first day in month
nOffset,	;Offset that the day in the week (of the 1st day in month) is from
		;Sunday
nSpacesB4,	;Number of spaces before 1st day of mth on row (there are 3 
		;spaces/day + 1 between each day)
nSpcsAft,	;Number of spaces from last day in month to end of cal
nDays,		;Number of days in month
weekNo.n        ;Number of week-of-year

d1stOfMth=dateval(strval(month(dNewDate))+"/01/"+strval(year(dNewDate)))
if mFirst.b then
   nOffSet=(search(dow(d1stOfMth),iif(iGer.b,"MonDieMitDonFreSamSon","MonTueWedThuFriSatSun"))+2)/3
else
   nOffSet=(search(dow(d1stOfMth),iif(iGer.b,"SonMonDieMitDonFreSam","SunMonTueWedThuFriSat"))+2)/3
endif   
nSpacesB4=(nOffset-1)*3
nDays=iif( month(dNewDate)=12, 31, 
           day(dateval(strval(month(dNewDate)+1)+"/1/"+strval(year(dNewDate)))-1) )
;Determine number of rows calendar will require
nCalRows=iif( mod(nSpacesB4+3*nDays,21)=0, (nSpacesB4+3*nDays)/21, int((nSpacesB4+3*nDays)/21)+1 )
;Build text string consisting of all days in month, padded in front and rear
;with spaces based on which days of the week the month starts and ends on.
a=spaces(nSpacesB4)
for i from 1 to nDays
	a=a+format("W3,AR",i)
endfor
;Determine, then add spaces after last day
nSpcsAft=0
if mFirst.b then
   nSpcsAft=(search(dow(d1stOfMth+nDays-1),iif(iGer.b,"SamFreDonMitDieMon","SatFriThuWedTueMon"))+2)
else
   nSpcsAft=(search(dow(d1stOfMth+nDays-1),iif(iGer.b,"FreDonMitDieMonSun","FriThuWedTueMonSun"))+2)
endif
a=a+spaces(nSpcsAft)
;Slice text string into formatted dynarray elements
for i from 1 to nCalRows
    DYCAL[i]=substr(a,i*21-20,21)
endfor

if wDisp.b then
   rkWeekOfYear(d1stOfMth)  ;creates array woy.ay
else
   array woy.ay[6]          ;another blind array..
   for i from 1 to 6
       woy.ay[i] = ""
   endfor
endif   

ENDPROC


PROC NewDay(dNewDate)
;=============================================================================
;Copyright 1992 Professional Information Consulting, Inc., Leon Chalnick
;You are granted the right to use this code in your applications as long as 
;this notice is included.  You are not granted the right to resell this code.
;=============================================================================
;AUTHOR:		PIC-LMC
;DATE:		8/27/92
;FUNCTION:
;	Determines screen coordinates of new date on calendar in dbox.
;	Note that the var, nCRow is used first to identify the array element in 
;DYCAL[] that contains the new day, then it's adjusted to represent the
;correct screen row for that day.
;
;PARAMS:
;	dNewDate  - The new calendar date for which cursor coords must be determined
;CALLED BY:
;	dbpGet2Date
;	SetUpCal
;	DecreaseDate
;	IncreaseDate
;=============================================================================
private
i			;Counter used in determining current cursor row and column

nOldRow=nCRow									;Save old positions so they can be
nOldCol=nCCol									;painted over.
;Determine cursor row for new date
for i from 1 to nCalRows
	;Check day of last number in array element (at end of week)
	if day(dNewDate)<=numval(substr(DYCAL[i],20,2))
	 or substr(DYCAL[i],20,2)="  " 
		then nCRow=i
		quitloop
	endif
endfor
;At this point, nCrow represents DYCAL[] array element containing day
;Determine cursor column for new date
for i from 1 to 7
	;Break row containing new day into 3 char chunks and compare to day
	if day(dNewDate)=numval(substr(DYCAL[nCRow],i*3-2,3))
		then nCCol=3+(i*3-2)					;Days start in column 3
		quitloop
	endif
endfor
nCRow=nCRow+1									;Add 1 cause dates start on 2nd row
;Now nCRow represents d-box _window row_ containing day
ENDPROC 

PROC DecreaseDate(aPeriod)
;=============================================================================
;Copyright 1992 Professional Information Consulting, Inc., Leon Chalnick
;You are granted the right to use this code in your applications as long as 
;this notice is included.  You are not granted the right to resell this code.
;=============================================================================
;AUTHOR:		PIC-LMC
;DATE:		8/27/92
;FUNCTION:
;	Decreases the current date (dCdate) by the time period specifed in aPeriod.
;This is either a DAY, WEEK, MONTH or YEAR.  Days and weeks are easy, because
;the number of days by which date will be changing is absolute.  The
;functionality of changing months and years is a little differnt though.
;	If user increments by a month (or year), I wanted new date to be set to the
;same numeric day (e.g., the 15th day of the mth) but in the next (or previous)
;month (or year).  In most cases, this isn't a problem.  However, not all
;months have 31 days.  So if, for example, cursor was on January 31 and user 
;hits PgUp for next mth, cursor will move to the LAST in February, whatever it
;happens to be.  The same problem exists when moving from year to year in the
;case of Febrary--you don't know how many days it has.
;  The way we check this is to create the new date by creating the new date
;with the current day.  If the resulting date="Error", then you know that we
;need to grab the last day of the month instead.
;	Of course, there's also the problem of changing years as you move back and
;forth between December and January.  This is handled easily with an iif().
;
;PARAMS:
;	aPeriod  - The period by which dCdate is to be decreased.  This is either
;DAY, WEEK, MONTH or YEAR
;
;CALLS:
;	NewMonth - Initializes vars used in producing calendar
;	NewDay   - Determines new cursor location for movment of one day
;CALLED BY:
;	dbpGet2Date
;=============================================================================
private
nNbrDays,		;Number of days by which date is to be changed
dOldDate,		;Original date.  Used to determine if month changed
nMth,			;New month; Makes date formula easier to read (a little <G>)
nYr			;New year; Makes date formula easier to read (a little <G>)

dOldDate=dCdate
switch
case aPeriod="DAY":
	dCdate=dCdate-1
case aPeriod="WEEK":
	dCdate=dCdate-7
case aPeriod="MONTH":
	nYr=iif(month(dCdate)=1, year(dCdate)-1, year(dCdate))
	nMth=iif(month(dCdate)=1, 12, month(dCdate)-1)
	;To determine last day of pervious month, will subtract 1 from 1st day of
	;current month.
	if "Error"=dateval(strval(nMth) +"/"+strval(day(dCdate))+"/"+ strval(nYr))
		then dCdate=dateval( strval(month(dCdate)) +"/1/"+ strval(year(dCdate)) )-1
	else
		dCdate=dateval(strval(nMth) +"/"+strval(day(dCdate))+"/"+ strval(nYr))
	endif
case aPeriod="YEAR":
	nYr=max(year(dCdate)-1,0)				;Don't let year go negative!
	;Logic below is same for MONTH, above
	if "Error"=dateval(strval(month(dCdate))+"/"+strval(day(dCdate))+"/"+strval(nYr))
		then nMth=iif(month(dCdate)=12, 1, month(dCdate)+1)
		dCdate=dateval( strval(nMth) +"/1/"+ strval(nYr) )-1
	else
		dCdate=dateval(strval(month(dCdate))+"/"+strval(day(dCdate))+"/"+strval(nYr))
	endif
endswitch
;Compare original date to new date to see if month changed.
if month(dOldDate)>month(dCdate)
 or month(dOldDate)=1 and month(dCdate)=12
 or year(dOldDate)<>year(dCdate)
	then NewMonth(dCdate)
endif
;Reproduce aMth and aYr for 'mouse button' display.  Not 100% efficient doing
;this on every call, but the code is quite fast...
if iGer.b=tGer.b then
   aMth=moy(dCDate)
else
   aMth=iif(tGer.b,substr("JanFebMrAprMaiJunJulAugSepOktNovDez",month(dCDate)*3-2,3),
                   substr("JanFebMarAprMayJunJulAugSepOctNovDec",month(dCDate)*3-2,3))
endif   
aYr=strval(year(dCdate))
NewDay(dCdate)
dAcptDate=dCdate								;Resync value in accept control
rkCalcDates()

ENDPROC

PROC IncreaseDate(aPeriod)
;=============================================================================
;Copyright 1992 Professional Information Consulting, Inc., Leon Chalnick
;You are granted the right to use this code in your applications as long as 
;this notice is included.  You are not granted the right to resell this code.
;=============================================================================
;AUTHOR:		PIC-LMC
;DATE:		8/27/92
;FUNCTION:
;	Increases the current date (dCdate) by the time period specifed in aPeriod.
;This is either a DAY, WEEK, MONTH or YEAR.  It works just like DecreaseDate()
;but obviously, in the opposite direction, so to understand the issues that it
;has to deal with, read comments for DecreaseDate().  They were not combined
;into one proc because some of the number we have to deal with when increasing
;the date are actually different (e.g. look at the logic under MONTH type 
;procesing.)
;
;PARAMS:
;	aPeriod  - The period by which dCdate is to be increased.  This is either
;DAY, WEEK, MONTH or YEAR
;
;CALLS:
;	NewMonth - Initializes vars used in producing calendar
;	NewDay   - Determines new cursor location for movment of one day
;CALLED BY:
;	dbpGet2Date
;=============================================================================
private
nNbrDays,		;Number of days by which date is to be changed
dOldDate,		;Original date.  Used to determine if month changed
nMth,				;New month; Makes date formula easier to read (a little <G>)
nYr				;New year; Makes date formula easier to read (a little <G>)

dOldDate=dCdate
switch
case aPeriod="DAY":
	dCdate=dCdate+1
case aPeriod="WEEK":
	dCdate=dCdate+7
case aPeriod="MONTH":
	nYr=iif(month(dCdate)=12, year(dCdate)+1, year(dCdate))
	nMth=iif(month(dCdate)=12, 1, month(dCdate)+1)
	;To determine last day of _Next_ month, must subtract 1 from 1st day of
	;two months from now.
	if "Error"=dateval(strval(nMth) +"/"+ strval(day(dCdate)) +"/"+ strval(nYr))
		then nMth=iif(month(dCdate)>=11, month(dCdate)-10, month(dCdate)+2)
		dCdate=dateval( strval(nMth) +"/1/"+ strval(year(dCdate)) )-1
	else
		dCdate=dateval( strval(nMth) +"/"+strval(day(dCdate))+"/"+ strval(nYr))
	endif
case aPeriod="YEAR":
	nYr=year(dCdate)+1
	if "Error"=
	 dateval(strval(month(dCdate)) +"/"+ strval(day(dCdate)) +"/"+ strval(nYr))
		then nMth=iif(month(dCdate)=12, 1, month(dCdate)+1)
		dCdate=dateval( strval(nMth) +"/1/"+ strval(nYr) )-1
	else
		dCdate=dateval( strval(month(dCdate)) 
		+"/"+ strval(day(dCdate))+"/"+ strval(nYr))
	endif
endswitch
;Compare original date to new date to see if month changed.
if month(dOldDate)<month(dCdate);Compare original date to new date
 or month(dOldDate)=12 and month(dCdate)=1
 or year(dOldDate)<>year(dCdate)
	then NewMonth(dCdate)
endif
;Reproduce aMth and aYr for 'mouse button' display.  Not 100% efficient doing
;this on every call, but the code is quite fast...
if iGer.b=tGer.b then
   aMth=moy(dCDate)
else
   aMth=iif(tGer.b,substr("JanFebMrAprMaiJunJulAugSepOktNovDez",month(dCDate)*3-2,3),
                   substr("JanFebMarAprMayJunJulAugSepOctNovDec",month(dCDate)*3-2,3))
endif   
aYr=strval(year(dCdate))
NewDay(dCdate)
dAcptDate=dCdate	;Resync value in accept control

if aPeriod<>"DAY" then      ;Although dOldDate is private anyway, it's used
   release vars dOldDate    ;to determine whether a day is added (only when 
endif                       ;aPeriod="DAY" and cursor right has been pressed) 
                            ;or subtracted in proc rkCalcDates
rkCalcDates()
;refreshdialog          ;done by rkCalcDates
ENDPROC


PROC rkSyncCal()
;====================================================================
;Author: Reinhard Kalinke, 8/02/95
;Params: none
;Private: none
;Function: Synchronizes calendar if date is entered to ACCEPT,
;          or if dAcptDate has been changed in rkCalcDates
;====================================================================

dCdate=dAcptDate
if iGer.b=tGer.b then
   aMth=moy(dCDate)
else
   aMth=iif(tGer.b,substr("JanFebMrAprMaiJunJulAugSepOktNovDez",month(dCDate)*3-2,3),
                   substr("JanFebMarAprMayJunJulAugSepOctNovDec",month(dCDate)*3-2,3))
endif   
aYr=strval(year(dCdate))
NewMonth(dCdate)
NewDay(dCdate)
if isassigned(hDays.dy) then
   rkGetHolidayPos()
endif   
   
ENDPROC


PROC rkGetHolidayPos()
;====================================================================
;Author: Reinhard Kalinke, 8/04/95
;Params: none
;Function: gets holidays for the current month and calculates
;          dBox-positions for painting them red
;====================================================================
private hMth,       ; Month of current Date 
        hYr,        ; Year of current Date 
        element,    ; Array-operator
        i,          ; Counter
        hCRow,      ; temp-Row 
        hCCol       ; temp-Col

array hPos.ay[10] ; initializing array for up to five holidays per month
for i from 1 to 10    
    hpos.ay[i] = 0    
endfor                

if isassigned(dAcptDate) then
   hMth = month(dAcptDate)
   hYr = year(dAcptDate)
else
   hMth = month(dDfltDate)   
   hYr = year(dDfltDate)
endif   

i = 1
foreach element in hDays.dy
  if month(dateval(element)) = hMth and year(dateval(element)) = hYr then
    rkHoliDay(dateval(element))  ; calculates hCRow and hCCol
    hPos.ay[i] = hCRow
    i = i+1
    hPos.ay[i] = hCCol
    i = i+1
    if i >= 10 then
       message iif(tGer.b,"Zu viele Feiertage in diesem Monat, "
                         +"Anzeige ist nicht vollstndig. "
                         +"Bitte verstndigen Sie den Softwarekasper!",
                          "Too many holidays this month, "
                         +"display is incomplete. "
                         +"Please consult your developer")
       sleep 3000
    endif   
  endif
endforeach    
    
ENDPROC

PROC rkHoliDay(dHoliDate.d) ;variation of NewDay()
;=============================================================================
;Copyright 1992 Professional Information Consulting, Inc., Leon Chalnick
;You are granted the right to use this code in your applications as long as 
;this notice is included.  You are not granted the right to resell this code.
;=============================================================================
;AUTHOR:		PIC-LMC, rk
;DATE:		8/27/92, 8/04/95
;FUNCTION:
;	Determines screen coordinates of holiday date on calendar in dbox.
;	Note that the var, hCRow is used first to identify the array element in 
;DYCAL[] that contains the new day, then it's adjusted to represent the
;correct screen row for that day.
;
;PARAMS:
;	dHoliDate.d  - The holiday date for which cursor coords must be determined
;CALLED BY:
;       rkGetHolidayPos
;=============================================================================
private
i	;Counter used in determining current cursor row and column

;Determine cursor row for new date
for i from 1 to nCalRows
	;Check day of last number in array element (at end of week)
	if day(dHoliDate.d)<=numval(substr(DYCAL[i],20,2))
	 or substr(DYCAL[i],20,2)="  " 
		then hCRow=i
		quitloop
	endif
endfor
;At this point, hCRow represents DYCAL[] array element containing day
;Determine cursor column for this date
for i from 1 to 7
	;Break row containing new day into 3 char chunks and compare to day
	if day(dHoliDate.d)=numval(substr(DYCAL[hCRow],i*3-2,3))
		then hCCol=3+(i*3-2)	;Days start in column 3
		quitloop
	endif
endfor
hCRow=hCRow+1									;Add 1 cause dates start on 2nd row
;Now hCRow represents d-box _window row_ containing day

ENDPROC 


Proc rkDowNum(Dte.d)   
;==============================================================
;Author: Reinhard Kalinke, 8/20/95
;Params: Dte.d, given date
;Function: returns dow as numeric value, 1=Monday if mFirst.b
;          version, 1=Sunday if not mFirst.b
;          uses an approach from CONVLIB.ZIP by Gary Samuelson
;called by: rkWeekOfYr
;==============================================================
private DWeek.n    ;holds return value

 DWeek.n = Mod((Dte.d-iif(mFirst.b,1/1/1500,1/1/1505)),7)+1    
           ;1/1/1500 was a monday, 1/1/1505 a sunday
 if DWeek.n < 1 then      ;though this is hardly gonna happen...
    Return 7+DWeek.n
 endif
 Return DWeek.n

ENDPROC


PROC rkWeekOfYear(d1st.d)
;============================================================
;Author: Reinhard Kalinke, 8/02/95
;Params: d1st.d, 1st day of current month
;Function: calculates the week-of-year as used in business schedule,
;          uses some snippets and ideas from CONVLIB.ZIP 
;          by Gary Samuelson
;called by: NewMonth
;============================================================
private
BegWk.n,      ;the week# the year begins with
EndWk.n,      ;the week# the year ends with
Wk2Dte.d,     ;the first day of 2nd week
weekNo.n      ;the week# of given date

switch
 case rkDowNum(dateval("1/1/"+strval(Year(d1st.d)))) > 5:
  BegWk.n = 52  ;year begins 52nd week when 1/1/ is Sat or Sun
 case rkDowNum(dateval("1/1/"+strval(Year(d1st.d)))) = 5:
  BegWk.n = 53  ;53rd week is added to previous year if it ends on Thursday
 otherwise:
  BegWk.n = 1
endswitch

if rkDowNum(dateval("1/1/"+strval(Year(d1st.d)))) < 5 then
  Wk2Dte.d = dateval(strval(9-rkDowNum(dateval("1/1/"+strval(Year(d1st.d)))))
            +".1."+strval(Year(d1st.d)))
else
  Wk2Dte.d = dateval(strval(16-rkDowNum(dateval("1/1/"+strval(Year(d1st.d)))))
            +".1."+strval(Year(d1st.d)))
endif

if d1st.d < Wk2Dte.d then
  if Wk2Dte.d - d1st.d <= 7 then
     weekNo.n = 1
  else
     weekNo.n = BegWk.n   
  endif 
else
  weekNo.n = 2+int((d1st.d-Wk2Dte.d)/7)
endif

array woy.ay[6]                ;array finally holding the display values

if month(d1st.d) = 1 and BegWk.n <> 1 then
   woy.ay[1] = format("w2,ar",strval(BegWk.n))
   for i from 2 to 6  ;max.# of nCalRows
       woy.ay[i] = format("w2,ar",strval(i-1))
   endfor
else
   for i from 1 to 6  ;max.# of nCalRows
       woy.ay[i] = format("w2,ar",strval(weekNo.n+i-1))
   endfor
endif

if month(d1st.d) = 12 then               ;overwriting last array element
   switch
    case rkDowNum(dateval("31.12."+strval(Year(d1st.d)))) < 4:
     EndKW.n = 1
    case rkDowNum(dateval("31.12."+strval(Year(d1st.d)))) = 4:
     EndKW.n = 53
    otherwise:
     EndKW.n = 52
   endswitch
   woy.ay[nCalRows] = format("w2,ar",strval(EndKw.n))
endif

ENDPROC


PROC rkCalcDates()
;====================================================================
;Author: Reinhard Kalinke, 8/02/95
;Params: none
;Function: denies a delivery date earlier than today()+1,
;          denies sundays and public holidays, calculates reminder
;
;          Behaviour of this proc maybe has to be customized
;          to suit your needs. This one has been developed for a 
;          customer who wants to determine delivery dates for 
;          incoming goods and wants to check they're on their way
;          so he can resell them to his customers in time.
;====================================================================
Private rDchanged.b,    ; true if dAcptDate is changed in this proc
        Mesg.a,         ; message text
        i,              ; Counter for adding/subtracting days
        j               ; Counter for displaying messages

rDchanged.b = false
Mesg.a = ""

;processing date:
i=1
j=1

while dow(dAcptDate) = iif(iGer.b,"Son","Sun") 
or dow(dAcptDate) = iif(iGer.b,"Sam","Sat")   ;disable if Sat included
or isassigned(hDays.dy[dAcptDate])          
or dAcptDate < today()+dateShiftMin.n 

   if dAcptDate < today()+dateShiftMin.n then
      if j=1 then
         beep
         Mesg.a = Mesg.a+iif(tGer.b,"Ein Termin vor dem "+strval(today()+dateShiftMin.n)
                                    +" kann nicht gewhlt werden",
                                    "No date prior to "+strval(today()+dateShiftMin.n)
                                    +" can be specified")
         isMesg.b = true
      endif
      dAcptDate = today()+dateShiftMin.n
      rDchanged.b = true
   endif
   
   
   if dow(dAcptDate) = iif(iGer.b,"Son","Sun") 
   or dow(dAcptDate) = iif(iGer.b,"Sam","Sat") ;disable if Sat included
   or isassigned(hDays.dy[dAcptDate]) then     
      if j=1 then
         sound 440 30
         if isassigned(hDays.dy[dAcptDate]) then
            Mesg.a = iif(tGer.b,hDays.dy[dAcptDate]+"!",
                                "That's "+hDays.dy[dAcptDate]+"!")
         else    
            ;Mesg.a = iif(tGer.b,"Sonntags nie!",   ;enable if Sat included
            ;                    "Sundays are not allowed")
            Mesg.a = iif(tGer.b,"Doch nicht am Wochenende!",;disable if Sat included 
                                "Weekends are not allowed")
         endif   
      endif
      if isassigned(nKey) and         ;adding a day, user wants to move 
      nKey = asc("RIGHT") then        ;forth and otherwise gets stuck when 
         dAcptDate = dAcptDate+1      ;using cursor right 
      else                             
         dAcptDate = dAcptDate-1      ;subtracting a day 
                                      ;to stay 'on the safer side'
         if dAcptDate < today()+dateShiftMin.n then
            dAcptDate = today()+dateShiftMin.n+i
            i=i+1           
         endif              
      endif   
      rDchanged.b = true
   endif
   j=j+1
endwhile   

message Mesg.a

if rDchanged.b or (aYr<>oYr or aMth<>oMth) then
   rkSyncCal()                        ;refresh calender display setup
endif   

refreshdialog                         ;show the user what he/she has done

ENDPROC

;=============================================================================
;Code below is an example of how to call dGet1Date()

;TPC if isfile("tpconfig.sc") then  ; TurboPAL config, when run with TP
;TPC    play "tpconfig"             ; and you want a special config
;TPC endif                          

;play sdir()+"colors3x"             ; Good old cyan/white 3.x colors...
                                    ; enable if you want them...

;Set up message/text language 
;(also if tbl HOLIDAYS or FEIERTAG is used)
;(internal language (var iGer.b) is set up in proc SetUpCal)
;tGer.b = true      ; text and messages in german
tGer.b = false     ; text and messages in english

;Set up first day of week
mFirst.b = true    ; week starts on Monday
;mFirst.b = false   ; week starts on Sunday

;Set up display of week-of-year 
wDisp.b = true     ; week will be displayed
;wDisp.b = false    ; week will not be displayed

;Set up dateShiftMin.n
dateShiftMin.n = 1

showpulldown									;Clear out menu
endmenu
clear												;clear PAL canvas

getDate.d = dGet1Date(dateval(today()))

if not isblank(getDate.d) then
   message iif(tGer.b,"Termin: "+strval(getDate.d),
                      "Date: "+strval(getDate.d))
else
   message iif(tGer.b,"Abbruch..",
                      "Canceled..")
endif      
GetEvent Key "ALL" Mouse "DOWN" to x
release vars x, hDays.dy, hPos.ay, tGer.b, mFirst.b, wDisp.b
             
;-----global vars are:---------------------------------------------             
;getDate.d            -> to be processed
;dateShiftMin.n       -> to prevent choosing a date prior to
;                        today()+dateShiftMin.n (can be negative,
;                        of course..)
;------------------------------------------------------------------

